#ifndef SSL_H
#define SSL_H

/*
 * Edit History
 *
 * Date        Initials                     Change description
 *
 * 2/1/07      JZW                          added comments warning that cipher_list field should be 
 *                                          persistent memory. Since it is passed
 *                                          into SSL this keeps its memory from being reused
 *                                          Vantive 1224523
 */

#ifdef __cplusplus
extern "C"
{
#endif

#include "http_awsapi.h"

/*ad*
 * Certificate database handle returned by @link naSslCreateCertDb
 *
 * @external
 * @category SSL
 * @since 6.3
 *
 * @see @link naSslCreateCertDb
*ad*/
typedef void * NaSslCertDbHandle_t;


/*ad*
 * Certificate verification callback function called during the process of verifying
 * incoming certificates.
 *
 * @external
 * @category SSL
 * @since 6.3
 *
 * @param sock_fd       Socket returned from previous call to accept
 * @param error         Error detected during certificate cerification. The following errors are reported:
 * <list type = "unordered">
 *      <item>SSL_ERR_CERT_NOT_YET_VALID    Certificate is not valid yet.</item> 
 *      <item>SSL_ERR_CERT_HAS_EXPIRED      Certificate is expired.</item>
 *      <item>SSL_ERR_CERT_REVOKED          Certificate has been revoked.</item>
 * </list>
 * @param cert          Pointer to the ASN.1 DER encoded certificate being verified.
 * @param len           Length, in bytes, of the certificate buffer pointed to by cert. 
 *
 * @return 1            The error is ignored and certificate verification continues.
 * @return 0            Forces certificate verification to fail. 
 *
 * @see @link naSslCreateCertDb
 *
*ad*/
typedef int (* NaSslCertVerifyCb_t)(int sock_fd, int error, const unsigned char * cert, int len);

/*ad*
 * The following structure is used to process incoming or outgoing SSLv3/TLSv1 connections 
 * using the certificate database. 
 *
 * @external
 * @category SSL
 * @since 6.3
 *
 * @param sock_fd               socket returned from previous call to accept()
 * @param cipher_list           cipher list to use for configuring ssl
 * @param cipher_list_len       length of the cipher list
 * @param cert_db_handle        handle to a certificate database
 *
 * @see @link naSslNgAccept
 * @see @link naSslNgConnect
 *
 * @note    Memory for cipher_list should be allocated by the application, but is retained by the 
 *          SSL thread.  Therefore this memory must be persistent for this SSL thread and unmodified 
 *          after calls to functions using this field.  More specifically, this should either be 
 *          malloc space, a global variable or local static storage.
*ad*/
typedef struct server_params
{
    int                 sock_fd;
    unsigned char       * cipher_list;
    int                 cipher_list_len;
    NaSslCertDbHandle_t cert_db_handle;
} NaSslConnParams_t;

/*ad*
 * Secure HTTP server parameters. 
 *
 * @external
 * @category SSL
 * @since 6.3
 *
 * @param cipher_list           cipher list to use for configuring ssl
 * @param cipher_list_len       length of the cipher list
 * @param secure_only           HTTPS_ONLY, or HTTP_AND_HTTPS
 * @param cert_db_handle        handle to a certificate database
 *
 * @see @link naHsStartServerSecure
 *
 * @note    Memory for cipher_list should be allocated by the application, but is retained by the 
 *          SSL thread.  Therefore this memory must be persistent for this SSL thread and unmodified 
 *          after calls to functions using this field.  More specifically, this should either be 
 *          malloc space, a global variable or local static storage.
*ad*/
typedef struct hs_server_params
{
    unsigned char * cipher_list;
    int           cipher_list_len;
    char          secure_only;
    NaSslCertDbHandle_t cert_db_handle;
} NaHsSrvSecConnParams_t;
    
/*ad*
 * This function creates a certificate database and returns its handle, which
 * must be passed into the other cetificate database functions. This function
 * must be called before using the certificate database. More than one instance
 * of a certificate database can be created by calling this function. 
 *
 * @external
 * @category SSL
 * @since 6.3
 * 
 * @return NULL     Certificate database could not be created.
 * @return non-NULL Certificate database handle.
 *
*ad*/
NaSslCertDbHandle_t naSslCreateCertDb(void);

/*ad*
 * This function adds certificates to the certificate database. More than one certificate may be present
 * in a PEM cert buffer at one time and the buffer does not have to persist after this function returns.
 * Only one identity certificate chain may be added and it must have a public/private key pair added as
 * well with a call to @link naSslAddCertKey. The identity certificate must be present all times since this
 * is what is sent to the remote host unless only the client mode is being used and certificate verification
 * has been disabled with the SSL_CERIFICATE_VERIFY_NONE flag in function @link naSslSetCertVerify. In this case,
 * the client does not send a certificate to the server even if a certificate request is received. The identity 
 * certificate may be self-signed and can be created with existing SSL function @link NASSLX509Generate.   
 *
 * @external
 * @category SSL
 * @since 6.3
 *
 * @param cert_db_handle  Certificate database handle.
 * @param cert      Pointer to a PEM or DER encoded certificate data buffer. More than one  certificate 
 *                  per buffer is allowed when the PEM format is used. OpenSSL's .pem files allow for more than
 *                  one certificate per file.   
 * @param cert_len  Length of certificate data.
 * @param cert_type There are five certificate types available: 
 * <list type = "unordered">
 *       <item>SSL_CERTIFICATE_CA           - Trusted Certificate Authority certificate.</item>
 *       <item>SSL_CERTIFICATE_IDENTITY     - Local identity certificate that is sent  to the remote host. 
 *                                            Only one instance of this certificate is permitted per database, however, more 
 *                                            than one certificate may be present in the Identity certificate chain
 *                                            ,pointed to by cert, if it uses OpenSSL's .pem file encoding.</item>
 *       <item>SSL_CERTIFICATE_CRL          - Certificate revocation list.</item>
 *       <item>SSL_CERTIFICATE_PEER         - Whitelist peer certificate.</item>
 *       <item>SSL_CERTIFICATE_PEER_REVOKED - Revoked peer certificate.</item>
 * </list>
 * @param cert_format There are two certificate formats supported:
 * <list type = "unordered">
 *       <item>SSL_CERTIFICATE_FORMAT_PEM - OpenSSL's .pem format.</item>
 *       <item>SSL_CERTIFICATE_FORMAT_DER - Standard ASN.1 DER.</item>
 * </list>
 * @param cert_id   User supplied nul-terminated string used to individually identify the certificate being added. 
 *                  This string can later be used to delete this certificate with a call to @link naSslDeleteCert.
 *                  It must not be longer than 32 bytes including the nul character. 
 *
 * 
 * @return SSL_SUCCESS                 Added certificate.
 * @return SSL_STATUS_INVALID_ARGUMENT One or more of the arguments is invalid.
 * @return SSL_STATUS_INTERNAL_ERROR   Error allocating resources.
 * @return SSL_STATUS_INVALID_CERT_ID  A certificate already exists with the same id string.
 *
*ad*/
int naSslAddCert(NaSslCertDbHandle_t cert_db_handle, void * cert, int cert_len, int cert_type, int cert_format, const char * cert_id);


/*ad*
 * This function deletes an individual certificate from the certificate database. If the Identity certificate was added
 * along with a certificate chain, the whole chain, including the Identity certificate, is deleted.
 *
 * @external
 * @category SSL
 * @since 6.3
 *
 * @param cert_db_handle  Certificate database handle.
 * @param cert_type There are five certificate types available: 
 * <list type = "unordered">
 *       <item>SSL_CERTIFICATE_CA           Trusted Certificate Authority certificate</item>
 *       <item>SSL_CERTIFICATE_IDENTITY     Local identity certificate that is sent to the remote host.</item>
 *       <item>SSL_CERTIFICATE_CRL          Certificate revocation list</item>
 *       <item>SSL_CERTIFICATE_PEER         Whitelist peer certificate</item>
 *       <item>SSL_CERTIFICATE_PEER_REVOKED Revoked peer certificate</item>
 * </list>
 * @param cert_id   User supplied nul-terminated string used to individually identify the certificate being deleted. 
 *                  This string must not be longer than 32 bytes including the nul character. 
 *
 * 
 * @return SSL_SUCCESS                 Deleted certificate.
 * @return SSL_STATUS_INVALID_ARGUMENT One or more of the arguments is invalid.
 * @return SSL_STATUS_INVALID_CERT_ID  Cannot locate certificate with the same string id.
 *
*ad*/
int naSslDeleteCert(NaSslCertDbHandle_t cert_db_handle, int cert_type, const char * cert_id);


/*ad*
 * This function adds a public/private key pair to the certificate database. Currently, only one RSA key pair can be added
 * to the database and it must always be present unless in client mode when certificates are not sent to the server by setting
 * the SSL_CERIFICATE_VERIFY_NONE flag in function @link naSslSetCertVerify. In this case, the client does not send a certificate
 * to the server even if a certificate request is received, hence there is not need for a private key.
 *
 * @external
 * @category SSL
 * @since 6.3
 *
 * @param cert_db_handle  Certificate database handle.
 * @param key       Pointer to the PEM or DER encoded key data buffer. Only one key pair is allowed in the PEM buffer.
 * @param key_len   Length of key buffer.
 * @param key_type  RSA is the only key type currently available - SSL_CERTIFICATE_KEY_TYPE_RSA.
 * @param key_format There are three key formats supported: 
 * <list type = "unordered">
 *       <item>SSL_CERTIFICATE_FORMAT_PEM  - Key pair is in PEM format.</item>
 *       <item>SSL_CERTIFICATE_FORMAT_DER  - Key pair is in DER format.</item>
 *       <item>SSL_CERTIFICATE_FORMAT_BIN  - Key pair is in binary format. This format should be used 
 *                                           with a key generated with @link NASSLX509Generate.</item>
 * </list>
 * @param pass_phrase  nul-terminated string used for decrypting encrypted private keys in PEM
 *                     format. If the key is not encrypted this parameter should be set to NULL .
 *
 * @return SSL_SUCCESS                 Success
 * @return SSL_STATUS_INVALID_ARGUMENT One or more of the arguments is invalid.
 * @return SSL_STATUS_INTERNAL_ERROR   Error allocating resources.
 *
*ad*/
int naSslAddCertKey(NaSslCertDbHandle_t cert_db_handle, void * key, int key_len, int key_type, int key_format, const char * pass_phrase);

/*ad*
 * This function sets various flags that dictate SSL's behavior during the handshake phase. A callback function
 * can also be specified to notify the user of various error conditions occurring during certificate verification.
 * See the definition of @link NaSslCertVerifyCb_t for usage of the callback. The individual flags are mutually exclusive
 * and cannot be OR'ed together.
 *
 * @external
 * @category SSL
 * @since 6.3
 *
 * @param cert_db_handle  Certificate database handle.
 * @param verify_cb       Certificate verification callback.
 * @param flags           The following flags are currently available:
 * <list type = "unordered">
 *       <item>SSL_CERTIFICATE_VERIFY_NONE - In server mode, no certificate request is sent to the client.
 *                                           In client mode, server certificates will not be verified.</item>
 *       <item>SSL_CERTIFICATE_VERIFY_PEER - In server mode, a certificate request is sent to the client; however,
 *                                           the handshake does not fail if the client does not send a certificate. 
 *                                           In client mode, the server certificate is verified. This is the default
 *                                           setting. </item>
 *       <item>SSL_CERTIFICATE_VERIFY_FAIL_IF_NO_PEER - In server mode, the handshake is terminated if the client 
 *                                           does not send a certificate after a certificate request is sent. This 
 *                                           flag has no meaning in client mode.</item>
 * </list>
 *
 * @return SSL_SUCCESS                 Success
 * @return SSL_STATUS_INVALID_ARGUMENT One or more of the arguments is invalid.
 *
 * @see @link NaSslCertVerifyCb_t
*ad*/
int naSslSetCertVerify(NaSslCertDbHandle_t cert_db_handle, NaSslCertVerifyCb_t verify_cb, int flags);

int NASSLAccept(int sockfd,
                unsigned char* cipher_list,
                int cipher_list_len,
                char* certificate,
                int certificate_len,
                char* key, int key_len);

int naSslNgAccept(NaSslConnParams_t * server_params);

int NASSLConnect(int sockfd, unsigned char* cipher_list, int cipher_list_len);

int naSslNgConnect(NaSslConnParams_t * client_params);


int NASSLX509Generate(char** certificate_data,
                      char** key_data,
                      int* key_data_len,
                      int certificate_key_type, 
                      int certificate_key_size, 
                      int serial_number, 
                      char* name_sub,
                      char* not_before_utc,
                      char* not_after_utc);

int HSStartServerSecure(unsigned char* cipher_list, 
                        int cipher_list_len, 
                        char* certificate, 
                        int certificate_len, 
                        char* key, 
                        int key_len,
                        char secure_only);

int naHsStartServerSecure(NaHsSrvSecConnParams_t * server_params);

int NARpStartServerSecure(RpHsSecureParams_t  * ParamsList);

int naRpNgStartServerSecure(NaRpHsSecureParams_t * ParamsList);

void naSslSetProxyPortNumber(unsigned short proxyPortNumber);

/*ad*
 * SSL and HTTPS return codes
 *
 * These are the completion codes returned by the SSL and HTTPS API.
 *
 * @external
 * @category SSL
 * @name sslCcodes    "SSL Return Codes"
 * @since 6.0
 *
 * @param SSL_SUCCESS                   Function completed successfully.
 * @param SSL_STATUS_INVALID_ARGUMENT	Invalid argument.
 * @param SSL_STATUS_UNSUPPORTED_CIPHER	Unsupported cipher suite.
 * @param SSL_STATUS_HANDSHAKE_FAILED	Unable to negotiate secure session.
 * @param SSL_STATUS_INTERNAL_ERROR		Error allocating resources.
 * @param SSL_STATUS_INVALID_CERT_ID    Certificate already exists with the same name when adding
 *                                      or one cannot be found with this name when deleting.
 * @param SSL_STATUS_CERTIFICATE_NOT_FOUND Certificate not in database. 
*ad*/
#define SSL_SUCCESS                      0
#define SSL_STATUS_INVALID_ARGUMENT		 -1	
#define SSL_STATUS_UNSUPPORTED_CIPHER	 -2
#define SSL_STATUS_HANDSHAKE_FAILED		 -3
#define SSL_STATUS_INTERNAL_ERROR		 -4
#define SSL_STATUS_CERTIFICATE_NOT_FOUND -5
#define SSL_STATUS_INVALID_CERT_ID       -6

#define SSL_CERTIFICATE_KEY_TYPE_RSA	1

#define HTTPS_ONLY		1
#define HTTP_AND_HTTPS	0

/* Supported Cipher Suites */
#define SSL_CIPHER_SUITE_TLS_RSA_WITH_DES_CBC_SHA               0x00,0x09
#define SSL_CIPHER_SUITE_TLS_RSA_WITH_3DES_EDE_CBC_SHA          0x00,0x0A
#define SSL_CIPHER_SUITE_TLS_RSA_WITH_AES_256_CBC_SHA			0x00,0x35
#define SSL_CIPHER_SUITE_TLS_RSA_WITH_AES_128_CBC_SHA			0x00,0x2F

#define SSL_CIPHER_LIST_HTTPS_DEFAULT		{ SSL_CIPHER_SUITE_TLS_RSA_WITH_3DES_EDE_CBC_SHA, SSL_CIPHER_SUITE_TLS_RSA_WITH_DES_CBC_SHA }
#define SSL_CIPHER_LIST_AES_DEFAULT			{ SSL_CIPHER_SUITE_TLS_RSA_WITH_AES_128_CBC_SHA, SSL_CIPHER_SUITE_TLS_RSA_WITH_AES_256_CBC_SHA }
#define SSL_CIPHER_LIST_DES_DEFAULT			{ SSL_CIPHER_SUITE_TLS_RSA_WITH_3DES_EDE_CBC_SHA, SSL_CIPHER_SUITE_TLS_RSA_WITH_DES_CBC_SHA }
#define SSL_CIPHER_LIST_ALL_DEFAULT			{ SSL_CIPHER_SUITE_TLS_RSA_WITH_3DES_EDE_CBC_SHA, SSL_CIPHER_SUITE_TLS_RSA_WITH_DES_CBC_SHA, SSL_CIPHER_SUITE_TLS_RSA_WITH_AES_128_CBC_SHA, SSL_CIPHER_SUITE_TLS_RSA_WITH_AES_256_CBC_SHA }
#define SSL_CIPHER_LIST_HTTPS_DEFAULT_LEN	4
#define SSL_CIPHER_LIST_AES_DEFAULT_LEN		4
#define SSL_CIPHER_LIST_DES_DEFAULT_LEN		4
#define SSL_CIPHER_LIST_ALL_DEFAULT_LEN		8    

int NASSLGetRemoteConnectionInformation (unsigned short localPort, unsigned long * RemoteIP, unsigned short * RemotePort);

#define NASSLGETPROXYINFOSUCCESS            0
#define NASSLGETPROXYINFOFAILURE            -1
#define NASSLGETPROXYINFOIPERROR            -2
#define NASSLGETPROXYINTOPORTERROR          -3

#define SSL_CERTIFICATE_VERIFY_PEER             0x01
#define SSL_CERTIFICATE_VERIFY_FAIL_IF_NO_PEER  0x02
#define SSL_CERTIFICATE_VERIFY_NONE             0x04

#define SSL_CERTIFICATE_FORMAT_PEM     10
#define SSL_CERTIFICATE_FORMAT_DER     11
#define SSL_CERTIFICATE_FORMAT_BIN     12

#define SSL_CERTIFICATE_REVOKED_PEER    2
#define SSL_CERTIFICATE_PEER            3
#define SSL_CERTIFICATE_CA              4
#define SSL_CERTIFICATE_CRL             5
#define SSL_CERTIFICATE_IDENTITY        6

#define SSL_ERR_CERT_NOT_YET_VALID 9
#define SSL_ERR_CERT_HAS_EXPIRED   10
#define SSL_ERR_CERT_REVOKED       23

#ifdef __cplusplus
}
#endif

#endif



